# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import netaddr

from tempest.common import utils
from tempest.common.utils import net_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class RouterRbacTest(base.BaseNetworkRbacTest):
    @classmethod
    def skip_checks(cls):
        super(RouterRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('router', 'network'):
            msg = "router extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(RouterRbacTest, cls).resource_setup()
        # Create a network with external gateway so that
        # ``external_gateway_info`` policies can be validated.
        post_body = {'router:external': True}
        cls.network = cls.create_network(**post_body)
        cls.subnet = cls.create_subnet(cls.network)
        cls.ip_range = netaddr.IPRange(
            cls.subnet['allocation_pools'][0]['start'],
            cls.subnet['allocation_pools'][0]['end'])
        cls.router = cls.create_router()

    def _get_unused_ip_address(self):
        unused_ip = net_utils.get_unused_ip_addresses(self.ports_client,
                                                      self.subnets_client,
                                                      self.network['id'],
                                                      self.subnet['id'],
                                                      1)
        return unused_ip[0]

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_router")
    @decorators.idempotent_id('acc5005c-bdb6-4192-bc9f-ece9035bb488')
    def test_create_router(self):
        """Create Router

        RBAC test for the neutron create_router policy
        """
        with self.rbac_utils.override_role(self):
            router = self.routers_client.create_router()
        self.addCleanup(self.routers_client.delete_router,
                        router['router']['id'])

    @decorators.idempotent_id('6139eb97-95c0-40d8-a109-99de11ab2e5e')
    @utils.requires_ext(extension='l3-ha', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_router:ha")
    def test_create_high_availability_router(self):
        """Create high-availability router

        RBAC test for the neutron create_router:ha policy
        """
        with self.rbac_utils.override_role(self):
            router = self.routers_client.create_router(ha=True)
        self.addCleanup(self.routers_client.delete_router,
                        router['router']['id'])

    @decorators.idempotent_id('c6254ca6-2728-412d-803d-d4aa3935e56d')
    @utils.requires_ext(extension='dvr', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_router:distributed")
    def test_create_distributed_router(self):
        """Create distributed router

        RBAC test for the neutron create_router:distributed policy
        """
        with self.rbac_utils.override_role(self):
            router = self.routers_client.create_router(distributed=True)
        self.addCleanup(self.routers_client.delete_router,
                        router['router']['id'])

    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rule="create_router:external_gateway_info:enable_snat")
    @decorators.idempotent_id('3c5acd49-0ec7-4109-ab51-640557b48ebc')
    def test_create_router_enable_snat(self):
        """Create Router Snat

        RBAC test for the neutron
        create_router:external_gateway_info:enable_snat policy
        """
        name = data_utils.rand_name(self.__class__.__name__ + '-snat-router')
        external_gateway_info = {'network_id': self.network['id'],
                                 'enable_snat': True}

        with self.rbac_utils.override_role(self):
            router = self.routers_client.create_router(
                name=name, external_gateway_info=external_gateway_info)
        self.addCleanup(self.routers_client.delete_router,
                        router['router']['id'])

    @rbac_rule_validation.action(
        service="neutron",
        rule="create_router:external_gateway_info:external_fixed_ips")
    @decorators.idempotent_id('d0354369-a040-4349-b869-645c8aed13cd')
    def test_create_router_external_fixed_ips(self):
        """Create Router Fixed IPs

        RBAC test for the neutron
        create_router:external_gateway_info:external_fixed_ips policy
        """
        name = data_utils.rand_name(self.__class__.__name__ + '-snat-router')

        unused_ip = self._get_unused_ip_address()
        external_fixed_ips = {'subnet_id': self.subnet['id'],
                              'ip_address': unused_ip}
        external_gateway_info = {'network_id': self.network['id'],
                                 'enable_snat': False,  # Default is True.
                                 'external_fixed_ips': [external_fixed_ips]}

        with self.rbac_utils.override_role(self):
            router = self.routers_client.create_router(
                name=name, external_gateway_info=external_gateway_info)
        self.addCleanup(self.routers_client.delete_router,
                        router['router']['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_router",
                                 expected_error_code=404)
    @decorators.idempotent_id('bfbdbcff-f115-4d3e-8cd5-6ada33fd0e21')
    def test_show_router(self):
        """Get Router

        RBAC test for the neutron get_router policy
        """
        # Prevent other policies from being enforced by using barebones fields.
        with self.rbac_utils.override_role(self):
            self.routers_client.show_router(self.router['id'], fields=['id'])

    @decorators.idempotent_id('3ed26ea2-b419-410c-b4b5-576c1edafa06')
    @utils.requires_ext(extension='dvr', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_router:distributed")
    def test_show_distributed_router(self):
        """Get distributed router

        RBAC test for the neutron get_router:distributed policy
        """
        router = self.routers_client.create_router(distributed=True)['router']
        self.addCleanup(self.routers_client.delete_router, router['id'])

        with self.rbac_utils.override_role(self):
            retrieved_fields = self.routers_client.show_router(
                router['id'], fields=['distributed'])['router']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if 'distributed' not in retrieved_fields:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='distributed')

    @decorators.idempotent_id('defc502c-4159-4824-b4d9-3cdcc39015b2')
    @utils.requires_ext(extension='l3-ha', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_router:ha")
    def test_show_high_availability_router(self):
        """GET high-availability router

        RBAC test for the neutron get_router:ha policy
        """
        router = self.routers_client.create_router(ha=True)['router']
        self.addCleanup(self.routers_client.delete_router, router['id'])

        with self.rbac_utils.override_role(self):
            retrieved_fields = self.routers_client.show_router(
                router['id'], fields=['ha'])['router']

        # Rather than throwing a 403, the field is not present, so raise exc.
        if 'ha' not in retrieved_fields:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='ha')

    @rbac_rule_validation.action(
        service="neutron", rule="update_router")
    @decorators.idempotent_id('3d182f4e-0023-4218-9aa0-ea2b0ae0bd7a')
    def test_update_router(self):
        """Update Router

        RBAC test for the neutron update_router policy
        """
        new_name = data_utils.rand_name(
            self.__class__.__name__ + '-new-router-name')
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(self.router['id'], name=new_name)

    @rbac_rule_validation.action(
        service="neutron", rule="update_router:external_gateway_info")
    @decorators.idempotent_id('5a6ae104-a9c3-4b56-8622-e1a0a0194474')
    def test_update_router_external_gateway_info(self):
        """Update Router External Gateway Info

        RBAC test for the neutron
        update_router:external_gateway_info policy
        """
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(self.router['id'],
                                              external_gateway_info={})

    @rbac_rule_validation.action(
        service="neutron",
        rule="update_router:external_gateway_info:network_id")
    @decorators.idempotent_id('f1fc5a23-e3d8-44f0-b7bc-47006ad9d3d4')
    def test_update_router_external_gateway_info_network_id(self):
        """Update Router External Gateway Info Network Id

        RBAC test for the neutron
        update_router:external_gateway_info:network_id policy
        """
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(
                self.router['id'],
                external_gateway_info={'network_id': self.network['id']})
        self.addCleanup(
            self.routers_client.update_router,
            self.router['id'],
            external_gateway_info=None)

    @utils.requires_ext(extension='ext-gw-mode', service='network')
    @rbac_rule_validation.action(
        service="neutron",
        rule="update_router:external_gateway_info:enable_snat")
    @decorators.idempotent_id('515a2954-3d79-4695-aeb9-d1c222765840')
    def test_update_router_enable_snat(self):
        """Update Router External Gateway Info Enable Snat

        RBAC test for the neutron
        update_router:external_gateway_info:enable_snat policy
        """
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(
                self.router['id'],
                external_gateway_info={'network_id': self.network['id'],
                                       'enable_snat': True})
        self.addCleanup(
            self.routers_client.update_router,
            self.router['id'],
            external_gateway_info=None)

    @rbac_rule_validation.action(
        service="neutron",
        rule="update_router:external_gateway_info:external_fixed_ips")
    @decorators.idempotent_id('f429e5ee-8f0a-4667-963e-72dd95d5adee')
    def test_update_router_external_fixed_ips(self):
        """Update Router External Gateway Info External Fixed Ips

        RBAC test for the neutron
        update_router:external_gateway_info:external_fixed_ips policy
        """
        unused_ip = self._get_unused_ip_address()
        external_fixed_ips = {'subnet_id': self.subnet['id'],
                              'ip_address': unused_ip}
        external_gateway_info = {'network_id': self.network['id'],
                                 'external_fixed_ips': [external_fixed_ips]}

        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(
                self.router['id'],
                external_gateway_info=external_gateway_info)
        self.addCleanup(
            self.routers_client.update_router,
            self.router['id'],
            external_gateway_info=None)

    @decorators.idempotent_id('ddc20731-dea1-4321-9abf-8772bf0b5977')
    @utils.requires_ext(extension='l3-ha', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_router:ha")
    def test_update_high_availability_router(self):
        """Update high-availability router

        RBAC test for the neutron update_router:ha policy
        """
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(self.router['id'], ha=True)
        self.addCleanup(self.routers_client.update_router, self.router['id'],
                        ha=False)

    @decorators.idempotent_id('e1932c19-8f73-41cd-b5d2-84c7ae5d530c')
    @utils.requires_ext(extension='dvr', service='network')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_router:distributed")
    def test_update_distributed_router(self):
        """Update distributed router

        RBAC test for the neutron update_router:distributed policy
        """
        with self.rbac_utils.override_role(self):
            self.routers_client.update_router(self.router['id'],
                                              distributed=True)
        self.addCleanup(self.routers_client.update_router, self.router['id'],
                        distributed=False)

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_router")
    @decorators.idempotent_id('c0634dd5-0467-48f7-a4ae-1014d8edb2a7')
    def test_delete_router(self):
        """Delete Router

        RBAC test for the neutron delete_router policy
        """
        router = self.create_router()
        with self.rbac_utils.override_role(self):
            self.routers_client.delete_router(router['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="add_router_interface")
    @decorators.idempotent_id('a0627778-d68d-4913-881b-e345360cca19')
    def test_add_router_interface(self):
        """Add Router Interface

        RBAC test for the neutron add_router_interface policy
        """
        network = self.create_network()
        subnet = self.create_subnet(network)
        router = self.create_router()

        with self.rbac_utils.override_role(self):
            self.routers_client.add_router_interface(
                router['id'], subnet_id=subnet['id'])
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.routers_client.remove_router_interface,
            router['id'],
            subnet_id=subnet['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="remove_router_interface")
    @decorators.idempotent_id('ff2593a4-2bff-4c27-97d3-dd3702b27dfb')
    def test_remove_router_interface(self):
        """Remove Router Interface

        RBAC test for the neutron remove_router_interface policy
        """
        network = self.create_network()
        subnet = self.create_subnet(network)
        router = self.create_router()

        self.routers_client.add_router_interface(
            router['id'], subnet_id=subnet['id'])

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.routers_client.remove_router_interface,
                        router['id'],
                        subnet_id=subnet['id'])

        with self.rbac_utils.override_role(self):
            self.routers_client.remove_router_interface(
                router['id'],
                subnet_id=subnet['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime

from six.moves.urllib import parse as urllib

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class InstanceUsagesAuditLogRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(InstanceUsagesAuditLogRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-instance-usage-audit-log',
                                          'compute'):
            msg = "os-instance-usage-audit-log extension not enabled."
            raise cls.skipException(msg)

    @decorators.idempotent_id('c80246c0-5c13-4ab0-97ba-91551cd53dc1')
    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-instance-usage-audit-log")
    def test_list_instance_usage_audit_logs(self):
        with self.rbac_utils.override_role(self):
            (self.instance_usages_audit_log_client
                .list_instance_usage_audit_logs())

    @decorators.idempotent_id('ded8bfbd-5d90-4a58-aee0-d31231bf3c9b')
    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-instance-usage-audit-log")
    def test_show_instance_usage_audit_log(self):
        now = datetime.datetime.now()

        with self.rbac_utils.override_role(self):
            (self.instance_usages_audit_log_client.
                show_instance_usage_audit_log(
                    urllib.quote(now.strftime("%Y-%m-%d %H:%M:%S"))))



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class ServerVolumeAttachmentRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def setup_clients(cls):
        super(ServerVolumeAttachmentRbacTest, cls).setup_clients()
        cls.volumes_client = cls.os_primary.volumes_client_latest

    @classmethod
    def skip_checks(cls):
        super(ServerVolumeAttachmentRbacTest, cls).skip_checks()
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def resource_setup(cls):
        super(ServerVolumeAttachmentRbacTest, cls).resource_setup()
        cls.server = cls.create_test_server(wait_until='ACTIVE')
        cls.volume = cls.create_volume()

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-volumes-attachments:index")
    @decorators.idempotent_id('529b668b-6edb-41d5-8886-d7dbd0614678')
    def test_list_volume_attachments(self):
        with self.rbac_utils.override_role(self):
            self.servers_client.list_volume_attachments(self.server['id'])

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-volumes-attachments:create")
    @decorators.idempotent_id('21c2c3fd-fbe8-41b1-8ef8-115ec47d54c1')
    def test_create_volume_attachment(self):
        with self.rbac_utils.override_role(self):
            self.attach_volume(self.server, self.volume)

    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-volumes-attachments:show")
    @decorators.idempotent_id('997df9c2-6e54-47b6-ab74-e4fdb500f385')
    def test_show_volume_attachment(self):
        attachment = self.attach_volume(self.server, self.volume)

        with self.rbac_utils.override_role(self):
            self.servers_client.show_volume_attachment(
                self.server['id'], attachment['id'])

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-volumes-attachments:update")
    @decorators.idempotent_id('bd667186-eca6-4b78-ab6a-3e2fabcb971f')
    def test_update_volume_attachment(self):
        attachment = self.attach_volume(self.server, self.volume)
        alt_volume = self.create_volume()

        with self.rbac_utils.override_role(self):
            self.servers_client.update_attached_volume(
                self.server['id'], attachment['id'], volumeId=alt_volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                alt_volume['id'], 'in-use')
        # On teardown detach the volume and wait for it to be available. This
        # is so we don't error out when trying to delete the volume during
        # teardown.
        self.addCleanup(waiters.wait_for_volume_resource_status,
                        self.volumes_client, alt_volume['id'], 'available')
        # Ignore 404s on detach in case the server is deleted or the volume
        # is already detached.
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.servers_client.detach_volume,
                        self.server['id'], alt_volume['id'])

    @decorators.attr(type='slow')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-volumes-attachments:delete")
    @decorators.idempotent_id('12b03e90-d087-46af-9c4d-507d021c4984')
    def test_delete_volume_attachment(self):
        self.attach_volume(self.server, self.volume)

        with self.rbac_utils.override_role(self):
            self.servers_client.detach_volume(self.server['id'],
                                              self.volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class CapabilitiesV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(CapabilitiesV3RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('capabilities', 'volume'):
            msg = "%s skipped as capabilities not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(CapabilitiesV3RbacTest, cls).setup_clients()
        cls.capabilities_client = cls.os_primary.volume_capabilities_v2_client
        cls.hosts_client = cls.os_primary.volume_hosts_v2_client

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume_extension:capabilities")
    @decorators.idempotent_id('40928b74-2141-11e7-93ae-92361f002671')
    def test_show_back_end_capabilities(self):
        host = self.hosts_client.list_hosts()['hosts'][0]['host_name']
        with self.rbac_utils.override_role(self):
            self.capabilities_client.show_backend_capabilities(host)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.image import rbac_base as base


class NamespaceTagsRbacTest(base.BaseV2ImageRbacTest):
    """RBAC tests for namespace_tags_client.

    Performs RBAC testing for the endpoints in namespace_tags_client, except
    for

        1) delete_namespace_tag
        2) delete_namespace_tags

    because Glance does not currently do policy enforcement for them.
    """

    @classmethod
    def resource_setup(cls):
        super(NamespaceTagsRbacTest, cls).resource_setup()
        cls.namespace = cls.namespaces_client.create_namespace(
            namespace=data_utils.rand_name(
                cls.__name__ + '-namespace'))['namespace']
        cls.addClassResourceCleanup(
            cls.namespaces_client.delete_namespace,
            cls.namespace)

    def _create_namespace_tag(self, multiple=False):
        tag_count = 2 if multiple else 1
        namespace_tag_names = []

        for i in range(tag_count):
            tag_name = data_utils.rand_name(self.__class__.__name__ + '-tag')
            namespace_tag_names.append({'name': tag_name})

        if multiple:
            namespace_tags = self.namespace_tags_client.create_namespace_tags(
                self.namespace, tags=namespace_tag_names)['tags']
        else:
            namespace_tags = self.namespace_tags_client.create_namespace_tag(
                self.namespace, namespace_tag_names[0]['name'])

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.namespace_tags_client.delete_namespace_tags,
                        self.namespace)

        return [nt['name'] for nt in namespace_tags] if multiple \
            else namespace_tags['name']

    @decorators.idempotent_id('50bedccb-9d0b-4138-8d95-31a89250edf6')
    @rbac_rule_validation.action(service="glance",
                                 rule="add_metadef_tag")
    def test_create_namespace_tag(self):
        with self.rbac_utils.override_role(self):
            self._create_namespace_tag()

    @decorators.idempotent_id('4acf70cc-05da-4b1e-87b2-d5e4475164e7')
    @rbac_rule_validation.action(service="glance",
                                 rule="get_metadef_tag")
    def test_show_namespace_tag(self):
        tag_name = self._create_namespace_tag()
        with self.rbac_utils.override_role(self):
            self.namespace_tags_client.show_namespace_tag(self.namespace,
                                                          tag_name)

    @decorators.idempotent_id('01593828-3edb-461e-8abc-8fdeb3927e37')
    @rbac_rule_validation.action(service="glance",
                                 rule="modify_metadef_tag")
    def test_update_namespace_tag(self):
        tag_name = self._create_namespace_tag()
        updated_tag_name = data_utils.rand_name(
            self.__class__.__name__ + '-tag')

        with self.rbac_utils.override_role(self):
            self.namespace_tags_client.update_namespace_tag(
                self.namespace, tag_name, name=updated_tag_name)

    @decorators.idempotent_id('20ffaf76-ebdc-4267-a1ad-194346f5cc91')
    @rbac_rule_validation.action(service="glance",
                                 rule="add_metadef_tags")
    def test_create_namespace_tags(self):
        with self.rbac_utils.override_role(self):
            self._create_namespace_tag(multiple=True)

    @decorators.idempotent_id('d37c1501-e787-449d-89b3-754a942a459a')
    @rbac_rule_validation.action(service="glance",
                                 rule="get_metadef_tags")
    def test_list_namespace_tags(self):
        with self.rbac_utils.override_role(self):
            self.namespace_tags_client.list_namespace_tags(self.namespace)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class BaseGroupRbacTest(rbac_base.BaseVolumeRbacTest):

    def setUp(self):
        super(BaseGroupRbacTest, self).setUp()
        self.volume_type_id = self.create_volume_type()['id']
        self.group_type_id = self.create_group_type()['id']

    def _create_group(self, name=None, ignore_notfound=False, **kwargs):
        group_name = name or data_utils.rand_name(
            self.__class__.__name__ + '-Group')
        group = self.groups_client.create_group(name=group_name, **kwargs)[
            'group']
        if ignore_notfound:
            self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                            self._delete_group, group['id'])
        else:
            self.addCleanup(self._delete_group, group['id'])
        waiters.wait_for_volume_resource_status(
            self.groups_client, group['id'], 'available')
        return group

    def _delete_group(self, group_id):
        self.groups_client.delete_group(group_id, delete_volumes=True)
        self.groups_client.wait_for_resource_deletion(group_id)

        vols = self.volumes_client.list_volumes(
            detail=True, params={'all_tenants': True})['volumes']
        for vol in vols:
            if vol['group_id'] == group_id:
                self.volumes_client.wait_for_resource_deletion(
                    vol['id'])


class GroupsV3RbacTest(BaseGroupRbacTest):
    min_microversion = '3.13'
    max_microversion = 'latest'

    @decorators.idempotent_id('43235328-66ae-424f-bc7f-f709c0ca268c')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:create")
    def test_create_group(self, name=None):

        group_name = name or data_utils.rand_name(
            self.__class__.__name__ + '-Group')
        with self.rbac_utils.override_role(self):
            group = self.groups_client.create_group(
                name=group_name, group_type=self.group_type_id,
                volume_types=[self.volume_type_id])['group']
        self.addCleanup(self._delete_group, group['id'])

        waiters.wait_for_volume_resource_status(
            self.groups_client, group['id'], 'available')

    @decorators.idempotent_id('9dc34a62-ae3e-439e-92b6-9389ea4c2863')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:get")
    def test_show_group(self):
        group = self._create_group(group_type=self.group_type_id,
                                   volume_types=[self.volume_type_id])

        with self.rbac_utils.override_role(self):
            self.groups_client.show_group(group['id'])

    @decorators.idempotent_id('db43841b-a173-4317-acfc-f83e4e48e4ee')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:get_all")
    def test_list_groups(self):
        with self.rbac_utils.override_role(self):
            self.groups_client.list_groups()['groups']

    @decorators.idempotent_id('5378da93-9c26-4ad4-b039-0555e2b8f668')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:get_all")
    def test_list_groups_with_details(self):
        with self.rbac_utils.override_role(self):
            self.groups_client.list_groups(detail=True)['groups']

    @decorators.idempotent_id('f499fc48-df83-4917-bf8d-783ebf6f080b')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:update")
    def test_update_group(self):
        group = self._create_group(group_type=self.group_type_id,
                                   volume_types=[self.volume_type_id])
        updated_name = data_utils.rand_name(self.__class__.__name__ + '-Group')

        with self.rbac_utils.override_role(self):
            self.groups_client.update_group(group['id'], name=updated_name)

    @decorators.idempotent_id('66fda391-5774-42a9-a018-80b34e57ab76')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:delete")
    def test_delete_group(self):

        group = self._create_group(ignore_notfound=True,
                                   group_type=self.group_type_id,
                                   volume_types=[self.volume_type_id])
        group_id = group['id']
        with self.rbac_utils.override_role(self):
            self.groups_client.delete_group(group_id, delete_volumes=True)

        self.groups_client.wait_for_resource_deletion(group_id)
        vols = self.volumes_client.list_volumes(
            detail=True, params={'all_tenants': True})['volumes']
        for vol in vols:
            if vol['group_id'] == group_id:
                self.volumes_client.wait_for_resource_deletion(
                    vol['id'])


class GroupV320RbacTest(BaseGroupRbacTest):
    _api_version = 3
    min_microversion = '3.20'
    max_microversion = 'latest'

    @decorators.idempotent_id('b849c1d4-3215-4f9d-b1e6-0aeb4b2b65ac')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:reset_status")
    def test_reset_group_status(self):
        group = self._create_group(ignore_notfound=False,
                                   group_type=self.group_type_id,
                                   volume_types=[self.volume_type_id])
        status = 'available'
        with self.rbac_utils.override_role(self):
            self.groups_client.reset_group_status(group['id'],
                                                  status)
        waiters.wait_for_volume_resource_status(
            self.groups_client, group['id'], status)


class GroupTypesV3RbacTest(rbac_base.BaseVolumeRbacTest):
    min_microversion = '3.11'
    max_microversion = 'latest'

    @decorators.idempotent_id('2820f12c-4681-4c7f-b28d-e6925637dff6')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:group_types_manage")
    def test_create_group_type(self):
        with self.rbac_utils.override_role(self):
            self.create_group_type(ignore_notfound=True)

    @decorators.idempotent_id('f77f8156-4fc9-4f02-be15-8930f748e10c')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:group_types_manage")
    def test_delete_group_type(self):
        group_type = self.create_group_type(ignore_notfound=True)

        with self.rbac_utils.override_role(self):
            self.group_types_client.delete_group_type(group_type['id'])

    @decorators.idempotent_id('67929954-4551-4d22-b15a-27fb6e56b711')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:group_types_manage")
    def test_update_group_type(self):
        group_type = self.create_group_type()
        update_params = {
            'name': data_utils.rand_name(
                self.__class__.__name__ + '-updated-group-type'),
            'description': 'updated-group-type-desc'
        }
        with self.rbac_utils.override_role(self):
            self.group_types_client.update_group_type(
                group_type['id'], **update_params)

    @decorators.idempotent_id('a5f88c26-df7c-4f21-a3ae-7a4c2d6212b4')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:access_group_types_specs")
    def test_create_group_type_group_specs(self):
        # TODO(felipemonteiro): Combine with ``test_create_group_type``
        # once multiple policy testing is supported. This policy is
        # only enforced after "group:group_types_manage".
        with self.rbac_utils.override_role(self):
            group_type = self.create_group_type(ignore_notfound=True)

        if 'group_specs' not in group_type:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='group_specs')

    @decorators.idempotent_id('8d9e2831-24c3-47b7-a76a-2e563287f12f')
    @rbac_rule_validation.action(
        service="cinder",
        rule="group:access_group_types_specs")
    def test_show_group_type(self):
        group_type = self.create_group_type()
        with self.rbac_utils.override_role(self):
            resp_body = self.group_types_client.show_group_type(
                group_type['id'])['group_type']
        if 'group_specs' not in resp_body:
            raise rbac_exceptions.RbacMalformedResponse(
                attribute='group_specs')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumesBasicCrudV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def resource_setup(cls):
        super(VolumesBasicCrudV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:create")
    @decorators.idempotent_id('426b08ef-6394-4d06-9128-965d5a6c38ef')
    def test_create_volume(self):
        with self.rbac_utils.override_role(self):
            self.create_volume()

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:delete")
    @decorators.idempotent_id('6de9f9c2-509f-4558-867b-af21c7163be4')
    def test_delete_volume(self):
        volume = self.create_volume()
        with self.rbac_utils.override_role(self):
            self.volumes_client.delete_volume(volume['id'])

    @rbac_rule_validation.action(service="cinder", rule="volume:get")
    @decorators.idempotent_id('c4c3fdd5-b1b1-49c3-b977-a9f40ee9257a')
    def test_get_volume(self):
        with self.rbac_utils.override_role(self):
            self.volumes_client.show_volume(self.volume['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:get_all")
    @decorators.idempotent_id('e3ab7906-b04b-4c45-aa11-1104d302f940')
    def test_volume_list(self):
        with self.rbac_utils.override_role(self):
            self.volumes_client.list_volumes()

    @rbac_rule_validation.action(service="cinder", rule="volume:update")
    @decorators.idempotent_id('b751b889-9a9b-40d8-ae7d-4b0f65e71ac7')
    def test_update_volume(self):
        update_name = data_utils.rand_name(self.__class__.__name__ + 'volume')
        with self.rbac_utils.override_role(self):
            self.volumes_client.update_volume(self.volume['id'],
                                              name=update_name)

    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:volume_image_metadata")
    @decorators.idempotent_id('3d48ca91-f02b-4616-a69d-4a8b296c8529')
    def test_volume_list_image_metadata(self):
        with self.rbac_utils.override_role(self):
            self.volumes_client.list_volumes(detail=True)



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityRoleAssignmentsV3RbacTest(rbac_base.BaseIdentityV3RbacTest):

    @decorators.idempotent_id('afe57adb-1b9c-43d9-84a9-f0cf4c94e416')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:list_role_assignments")
    def test_list_role_assignments(self):
        with self.rbac_utils.override_role(self):
            self.role_assignments_client.list_role_assignments()

    @decorators.idempotent_id('36c7a990-857e-415c-8717-38d7200a9894')
    @rbac_rule_validation.action(
        service="keystone",
        rule="identity:list_role_assignments_for_tree")
    def test_list_role_assignments_for_tree(self):
        project = self.setup_test_project()

        with self.rbac_utils.override_role(self):
            self.role_assignments_client.list_role_assignments(
                include_subtree=True,
                **{'scope.project.id': project['id']})



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import tempest_fixtures as fixtures
from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class QuotaClassesRbacTest(rbac_base.BaseV2ComputeRbacTest):

    def setUp(self):
        # All test cases in this class need to externally lock on doing
        # anything with default quota values.
        self.useFixture(fixtures.LockFixture('compute_quotas'))
        super(QuotaClassesRbacTest, self).setUp()

    @classmethod
    def skip_checks(cls):
        super(QuotaClassesRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-quota-class-sets', 'compute'):
            msg = "%s skipped as os-quota-class-sets extension not enabled."\
                  % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(QuotaClassesRbacTest, cls).setup_clients()
        cls.quota_classes_client = cls.os_primary.quota_classes_client
        cls.identity_projects_client = cls.os_primary.projects_client

    @classmethod
    def resource_setup(cls):
        super(QuotaClassesRbacTest, cls).resource_setup()
        # Create a project with its own quota.
        project_name = data_utils.rand_name(cls.__name__ + '-Project')
        cls.project_id = cls.identity_projects_client.create_project(
            project_name)['project']['id']
        cls.addClassResourceCleanup(
            cls.identity_projects_client.delete_project, cls.project_id)

    @decorators.idempotent_id('c10198ed-9df2-440e-a49b-367dadc6de94')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-quota-class-sets:show")
    def test_show_quota_class_set(self):
        with self.rbac_utils.override_role(self):
            self.quota_classes_client.show_quota_class_set('default')

    @decorators.idempotent_id('81889e69-efd2-4e96-bb4c-ee3b646b9755')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-quota-class-sets:update")
    def test_update_quota_class_set(self):
        # Update the pre-existing quotas for the project_id.
        quota_class_set = self.quota_classes_client.show_quota_class_set(
            self.project_id)['quota_class_set']
        quota_class_set.pop('id')
        for quota, default in quota_class_set.items():
            quota_class_set[quota] = default + 100

        with self.rbac_utils.override_role(self):
            self.quota_classes_client.update_quota_class_set(
                self.project_id, **quota_class_set)



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class HypervisorRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(HypervisorRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-hypervisors', 'compute'):
            msg = "%s skipped as os-hypervisors extension not enabled." \
                  % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(HypervisorRbacTest, cls).resource_setup()
        cls.hypervisor =\
            cls.hypervisor_client.list_hypervisors()['hypervisors'][0]

    @decorators.idempotent_id('17bbeb9a-e73e-445f-a771-c794448ef562')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_list_hypervisors(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.list_hypervisors()

    @decorators.idempotent_id('36b95c7d-1085-487a-a674-b7c1ca35f520')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_list_hypervisors_with_details(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.list_hypervisors(detail=True)

    @decorators.idempotent_id('8a7f6f9e-34a6-4480-8875-bba566c3a581')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_show_hypervisor(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.show_hypervisor(self.hypervisor['id'])

    @decorators.idempotent_id('b86f03cf-2e79-4d88-9eea-62f761591413')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_list_servers_on_hypervisor(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.list_servers_on_hypervisor(
                self.hypervisor['hypervisor_hostname'])

    @decorators.idempotent_id('ca0e465c-6365-4a7f-ae58-6f8ddbca06c2')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_show_hypervisor_statistics(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.show_hypervisor_statistics()

    @decorators.idempotent_id('109b37c5-91ba-4da5-b2a2-d7618d84406d')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_show_hypervisor_uptime(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.show_hypervisor_uptime(
                self.hypervisor['id'])

    @decorators.idempotent_id('3dbc71c1-8f04-4674-a67c-dcb2fd99b1b4')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-hypervisors")
    def test_search_hypervisor(self):
        with self.rbac_utils.override_role(self):
            self.hypervisor_client.search_hypervisor(
                self.hypervisor['hypervisor_hostname'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import mock
import os

from tempest import config
from tempest.tests import base

from patrole_tempest_plugin import policy_authority
from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin.tests.unit import fixtures

CONF = config.CONF


class PolicyAuthorityTest(base.TestCase):

    services = {
        'services': [
            {'name': 'custom_rbac_policy'},
            {'name': 'admin_rbac_policy'},
            {'name': 'alt_admin_rbac_policy'},
            {'name': 'tenant_rbac_policy'},
            {'name': 'test_service'}
        ]
    }

    def setUp(self):
        super(PolicyAuthorityTest, self).setUp()
        self.patchobject(policy_authority, 'credentials')
        m_creds = self.patchobject(policy_authority, 'clients')
        m_creds.Manager().identity_services_client.list_services.\
            return_value = self.services
        m_creds.Manager().identity_services_v3_client.list_services.\
            return_value = self.services

        current_directory = os.path.dirname(os.path.realpath(__file__))
        self.custom_policy_file = os.path.join(current_directory,
                                               'resources',
                                               'custom_rbac_policy.json')
        self.admin_policy_file = os.path.join(current_directory,
                                              'resources',
                                              'admin_rbac_policy.json')
        self.alt_admin_policy_file = os.path.join(current_directory,
                                                  'resources',
                                                  'alt_admin_rbac_policy.json')
        self.tenant_policy_file = os.path.join(current_directory,
                                               'resources',
                                               'tenant_rbac_policy.json')
        self.conf_policy_path = os.path.join(
            current_directory, 'resources', '%s.json')

        self.useFixture(fixtures.ConfPatcher(
            custom_policy_files=[self.conf_policy_path], group='patrole'))
        self.useFixture(fixtures.ConfPatcher(
            api_v3=True, api_v2=False, group='identity-feature-enabled'))

        # Guarantee a blank slate for each test.
        for attr in ('available_services', 'policy_files'):
            if attr in dir(policy_authority.PolicyAuthority):
                delattr(policy_authority.PolicyAuthority, attr)

    def _get_fake_policy_rule(self, name, rule):
        fake_rule = mock.Mock(check=rule, __name__='foo')
        fake_rule.name = name
        return fake_rule

    @mock.patch.object(policy_authority, 'LOG', autospec=True)
    def test_custom_policy(self, m_log):
        default_roles = ['zero', 'one', 'two', 'three', 'four',
                         'five', 'six', 'seven', 'eight', 'nine']

        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "custom_rbac_policy")

        expected = {
            'policy_action_1': ['two', 'four', 'six', 'eight'],
            'policy_action_2': ['one', 'three', 'five', 'seven', 'nine'],
            'policy_action_3': ['zero'],
            'policy_action_4': ['one', 'two', 'three', 'five', 'seven'],
            'policy_action_5': ['zero', 'one', 'two', 'three', 'four', 'five',
                                'six', 'seven', 'eight', 'nine'],
            'policy_action_6': ['eight'],
        }

        for rule, role_list in expected.items():
            for role in role_list:
                self.assertTrue(authority.allowed(rule, role))
            for role in set(default_roles) - set(role_list):
                self.assertFalse(authority.allowed(rule, role))

    def test_admin_policy_file_with_admin_role(self):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "admin_rbac_policy")

        role = 'admin'
        allowed_rules = [
            'admin_rule', 'is_admin_rule', 'alt_admin_rule'
        ]
        disallowed_rules = ['non_admin_rule']

        for rule in allowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertTrue(allowed)

        for rule in disallowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertFalse(allowed)

    def test_admin_policy_file_with_member_role(self):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "admin_rbac_policy")

        role = 'Member'
        allowed_rules = [
            'non_admin_rule'
        ]
        disallowed_rules = [
            'admin_rule', 'is_admin_rule', 'alt_admin_rule']

        for rule in allowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertTrue(allowed)

        for rule in disallowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertFalse(allowed)

    def test_alt_admin_policy_file_with_context_is_admin(self):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "alt_admin_rbac_policy")

        role = 'fake_admin'
        allowed_rules = ['non_admin_rule']
        disallowed_rules = ['admin_rule']

        for rule in allowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertTrue(allowed)

        for rule in disallowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertFalse(allowed)

        role = 'super_admin'
        allowed_rules = ['admin_rule']
        disallowed_rules = ['non_admin_rule']

        for rule in allowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertTrue(allowed)

        for rule in disallowed_rules:
            allowed = authority.allowed(rule, role)
            self.assertFalse(allowed)

    def test_tenant_user_policy(self):
        """Test whether rules with format tenant_id/user_id formatting work.

        Test whether Neutron rules that contain project_id, tenant_id, and
        network:tenant_id pass. And test whether Nova rules that contain
        user_id pass.
        """
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "tenant_rbac_policy")

        # Check whether Member role can perform expected actions.
        allowed_rules = ['rule1', 'rule2', 'rule3', 'rule4']
        for rule in allowed_rules:
            allowed = authority.allowed(rule, 'Member')
            self.assertTrue(allowed)

        disallowed_rules = ['admin_tenant_rule', 'admin_user_rule']
        for disallowed_rule in disallowed_rules:
            self.assertFalse(authority.allowed(disallowed_rule, 'Member'))

        # Check whether admin role can perform expected actions.
        allowed_rules.extend(disallowed_rules)
        for rule in allowed_rules:
            allowed = authority.allowed(rule, 'admin')
            self.assertTrue(allowed)

        # Check whether _try_rule is called with the correct target dictionary.
        with mock.patch.object(
            authority, '_try_rule', return_value=True, autospec=True) \
            as mock_try_rule:

            expected_target = {
                "project_id": mock.sentinel.tenant_id,
                "tenant_id": mock.sentinel.tenant_id,
                "network:tenant_id": mock.sentinel.tenant_id,
                "user_id": mock.sentinel.user_id
            }

            expected_access_data = {
                "roles": ['Member'],
                "is_admin": False,
                "is_admin_project": True,
                "user_id": mock.sentinel.user_id,
                "tenant_id": mock.sentinel.tenant_id,
                "project_id": mock.sentinel.tenant_id
            }

            for rule in allowed_rules:
                allowed = authority.allowed(rule, 'Member')
                self.assertTrue(allowed)
                mock_try_rule.assert_called_once_with(
                    rule, expected_target, expected_access_data, mock.ANY)
                mock_try_rule.reset_mock()

    @mock.patch.object(policy_authority, 'LOG', autospec=True)
    def test_invalid_service_raises_exception(self, m_log):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        service = 'invalid_service'

        self.assertRaises(rbac_exceptions.RbacInvalidService,
                          policy_authority.PolicyAuthority,
                          test_tenant_id,
                          test_user_id,
                          service)

        m_log.debug.assert_called_once_with(
            '%s is NOT a valid service.', service)

    @mock.patch.object(policy_authority, 'LOG', autospec=True)
    def test_service_is_none_raises_exception(self, m_log):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        service = None

        self.assertRaises(rbac_exceptions.RbacInvalidService,
                          policy_authority.PolicyAuthority,
                          test_tenant_id,
                          test_user_id,
                          service)

        m_log.debug.assert_called_once_with('%s is NOT a valid service.', None)

    @mock.patch.object(policy_authority, 'LOG', autospec=True)
    def test_invalid_policy_rule_throws_rbac_parsing_exception(self, m_log):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "custom_rbac_policy")

        fake_rule = 'fake_rule'
        expected_message = (
            "Policy action \"{0}\" not found in policy file: {1} or among "
            "registered policy in code defaults for service.").format(
            fake_rule, self.custom_policy_file)

        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              authority.allowed, fake_rule, None)
        self.assertIn(expected_message, str(e))
        m_log.debug.assert_called_once_with(expected_message)

    @mock.patch.object(policy_authority, 'LOG', autospec=True)
    def test_unknown_exception_throws_rbac_parsing_exception(self, m_log):
        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id

        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "custom_rbac_policy")
        authority.rules = mock.MagicMock(
            __name__='foo',
            **{'__getitem__.return_value.side_effect': Exception(
               mock.sentinel.error)})

        expected_message = (
            "Policy action \"{0}\" not found in policy file: {1} or among "
            "registered policy in code defaults for service.").format(
            mock.sentinel.rule, self.custom_policy_file)

        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              authority.allowed, mock.sentinel.rule, None)
        self.assertIn(expected_message, str(e))
        m_log.debug.assert_called_once_with(expected_message)

    @mock.patch.object(policy_authority, 'stevedore', autospec=True)
    def test_get_policy_data_from_file_and_from_code(self, mock_stevedore):
        fake_policy_rules = [
            self._get_fake_policy_rule('code_policy_action_1',
                                       'rule:code_rule_1'),
            self._get_fake_policy_rule('code_policy_action_2',
                                       'rule:code_rule_2'),
            self._get_fake_policy_rule('code_policy_action_3',
                                       'rule:code_rule_3'),
        ]

        mock_manager = mock.Mock(obj=fake_policy_rules, __name__='foo')
        mock_manager.configure_mock(name='fake_service')
        mock_stevedore.named.NamedExtensionManager.return_value = [
            mock_manager
        ]

        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id
        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, "tenant_rbac_policy")

        policy_data = authority._get_policy_data('fake_service')
        self.assertIsInstance(policy_data, str)

        actual_policy_data = json.loads(policy_data)
        expected_policy_data = {
            "code_policy_action_1": "rule:code_rule_1",
            "code_policy_action_2": "rule:code_rule_2",
            "code_policy_action_3": "rule:code_rule_3",
            "rule1": "tenant_id:%(network:tenant_id)s",
            "rule2": "tenant_id:%(tenant_id)s",
            "rule3": "project_id:%(project_id)s",
            "rule4": "user_id:%(user_id)s",
            "admin_tenant_rule": "role:admin and tenant_id:%(tenant_id)s",
            "admin_user_rule": "role:admin and user_id:%(user_id)s"
        }

        self.assertEqual(expected_policy_data, actual_policy_data)

    @mock.patch.object(policy_authority, 'stevedore', autospec=True)
    def test_get_policy_data_from_file_and_from_code_with_overwrite(
            self, mock_stevedore):
        # The custom policy file should overwrite default rules rule1 and rule2
        # that are defined in code.
        fake_policy_rules = [
            self._get_fake_policy_rule('rule1', 'rule:code_rule_1'),
            self._get_fake_policy_rule('rule2', 'rule:code_rule_2'),
            self._get_fake_policy_rule('code_policy_action_3',
                                       'rule:code_rule_3'),
        ]

        mock_manager = mock.Mock(obj=fake_policy_rules, __name__='foo')
        mock_manager.configure_mock(name='fake_service')
        mock_stevedore.named.NamedExtensionManager.return_value = [
            mock_manager
        ]

        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id

        authority = policy_authority.PolicyAuthority(
            test_tenant_id, test_user_id, 'tenant_rbac_policy')
        policy_data = authority._get_policy_data('fake_service')
        self.assertIsInstance(policy_data, str)

        actual_policy_data = json.loads(policy_data)
        expected_policy_data = {
            "code_policy_action_3": "rule:code_rule_3",
            "rule1": "tenant_id:%(network:tenant_id)s",
            "rule2": "tenant_id:%(tenant_id)s",
            "rule3": "project_id:%(project_id)s",
            "rule4": "user_id:%(user_id)s",
            "admin_tenant_rule": "role:admin and tenant_id:%(tenant_id)s",
            "admin_user_rule": "role:admin and user_id:%(user_id)s"
        }

        self.assertEqual(expected_policy_data, actual_policy_data)

    @mock.patch.object(policy_authority, 'stevedore', autospec=True)
    def test_get_policy_data_cannot_find_policy(self, mock_stevedore):
        mock_stevedore.named.NamedExtensionManager.return_value = None
        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              policy_authority.PolicyAuthority,
                              None, None, 'test_service')

        expected_error = \
            'Policy file for {0} service neither found in code '\
            'nor at {1}.'.format(
                'test_service',
                [CONF.patrole.custom_policy_files[0] % 'test_service'])

        self.assertIn(expected_error, str(e))

    @mock.patch.object(policy_authority, 'json', autospec=True)
    @mock.patch.object(policy_authority, 'stevedore', autospec=True)
    def test_get_policy_data_without_valid_policy(self, mock_stevedore,
                                                  mock_json):
        test_policy_action = mock.Mock(check='rule:bar', __name__='foo')
        test_policy_action.configure_mock(name='foo')

        test_policy = mock.Mock(obj=[test_policy_action], __name__='foo')
        test_policy.configure_mock(name='test_service')

        mock_stevedore.named.NamedExtensionManager\
            .return_value = [test_policy]

        mock_json.dumps.side_effect = ValueError

        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              policy_authority.PolicyAuthority,
                              None, None, 'test_service')

        expected_error = "Policy file for {0} service is invalid."\
                         .format("test_service")
        self.assertIn(expected_error, str(e))

        mock_stevedore.named.NamedExtensionManager.assert_called_once_with(
            'oslo.policy.policies',
            names=['test_service'],
            on_load_failure_callback=None,
            invoke_on_load=True,
            warn_on_missing_entrypoint=False)

    @mock.patch.object(policy_authority, 'json', autospec=True)
    @mock.patch.object(policy_authority, 'stevedore', autospec=True)
    def test_get_policy_data_from_file_not_json(self, mock_stevedore,
                                                mock_json):
        mock_stevedore.named.NamedExtensionManager.return_value = None
        mock_json.loads.side_effect = ValueError
        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              policy_authority.PolicyAuthority,
                              None, None, 'tenant_rbac_policy')

        expected_error = (
            'Policy file for {0} service neither found in code nor at {1}.'
            .format('tenant_rbac_policy', [CONF.patrole.custom_policy_files[0]
                                           % 'tenant_rbac_policy']))
        self.assertIn(expected_error, str(e))

    def test_discover_policy_files(self):
        policy_parser = policy_authority.PolicyAuthority(
            None, None, 'tenant_rbac_policy')

        # Ensure that "policy_files" is set at class and instance levels.
        self.assertIn('policy_files',
                      dir(policy_authority.PolicyAuthority))
        self.assertIn('policy_files', dir(policy_parser))
        self.assertIn('tenant_rbac_policy', policy_parser.policy_files)
        self.assertEqual(self.conf_policy_path % 'tenant_rbac_policy',
                         policy_parser.policy_files['tenant_rbac_policy'])

    @mock.patch.object(policy_authority, 'policy', autospec=True)
    @mock.patch.object(policy_authority.PolicyAuthority, '_get_policy_data',
                       autospec=True)
    @mock.patch.object(policy_authority, 'clients', autospec=True)
    @mock.patch.object(policy_authority, 'os', autospec=True)
    def test_discover_policy_files_with_many_invalid_one_valid(self, m_os,
                                                               m_creds, *args):
        # Only the 3rd path is valid.
        m_os.path.isfile.side_effect = [False, False, True, False]

        # Ensure the outer for loop runs only once in `discover_policy_files`.
        m_creds.Manager().identity_services_v3_client.\
            list_services.return_value = {
                'services': [{'name': 'test_service'}]}

        # The expected policy will be 'baz/test_service'.
        self.useFixture(fixtures.ConfPatcher(
            custom_policy_files=['foo/%s', 'bar/%s', 'baz/%s'],
            group='patrole'))

        policy_parser = policy_authority.PolicyAuthority(
            None, None, 'test_service')

        # Ensure that "policy_files" is set at class and instance levels.
        self.assertIn('policy_files',
                      dir(policy_authority.PolicyAuthority))
        self.assertIn('policy_files', dir(policy_parser))
        self.assertIn('test_service', policy_parser.policy_files)
        self.assertEqual('baz/test_service',
                         policy_parser.policy_files['test_service'])

    def test_discover_policy_files_with_no_valid_files(self):
        expected_error = ("Policy file for test_service service neither found "
                          "in code nor at %s." %
                          [self.conf_policy_path % 'test_service'])

        e = self.assertRaises(rbac_exceptions.RbacParsingException,
                              policy_authority.PolicyAuthority,
                              None, None, 'test_service')
        self.assertIn(expected_error, str(e))

        self.assertIn('policy_files',
                      dir(policy_authority.PolicyAuthority))
        self.assertNotIn(
            'test_service',
            policy_authority.PolicyAuthority.policy_files.keys())

    def _test_validate_service(self, v2_services, v3_services,
                               expected_failure=False, expected_services=None):
        with mock.patch.object(
            policy_authority, 'clients', autospec=True) as m_creds:
            m_creds.Manager().identity_services_client.list_services.\
                return_value = v2_services
            m_creds.Manager().identity_services_v3_client.list_services.\
                return_value = v3_services

        test_tenant_id = mock.sentinel.tenant_id
        test_user_id = mock.sentinel.user_id

        mock_os = self.patchobject(policy_authority, 'os')
        mock_os.path.join.return_value = self.admin_policy_file

        if not expected_services:
            expected_services = [s['name'] for s in self.services['services']]

        # Guarantee a blank slate for this test.
        if hasattr(policy_authority.PolicyAuthority, 'available_services'):
            delattr(policy_authority.PolicyAuthority,
                    'available_services')

        if expected_failure:
            policy_parser = None

            expected_exception = 'invalid_service is NOT a valid service'
            with self.assertRaisesRegex(rbac_exceptions.RbacInvalidService,
                                        expected_exception):
                policy_authority.PolicyAuthority(
                    test_tenant_id, test_user_id, "INVALID_SERVICE")
        else:
            policy_parser = policy_authority.PolicyAuthority(
                test_tenant_id, test_user_id, "tenant_rbac_policy")

        # Check that the attribute is available at object and class levels.
        # If initialization failed, only check at class level.
        if policy_parser:
            self.assertTrue(hasattr(policy_parser, 'available_services'))
            self.assertEqual(expected_services,
                             policy_parser.available_services)
        self.assertTrue(hasattr(policy_authority.PolicyAuthority,
                                'available_services'))
        self.assertEqual(
            expected_services,
            policy_authority.PolicyAuthority.available_services)

    def test_validate_service(self):
        """Positive test case to ensure ``validate_service`` works.

        There are 3 possibilities:
            1) Identity v3 API enabled.
            2) Identity v2 API enabled.
            3) Both are enabled.
        """
        self.useFixture(fixtures.ConfPatcher(
            api_v2=True, api_v3=False, group='identity-feature-enabled'))
        self._test_validate_service(self.services, [], False)

        self.useFixture(fixtures.ConfPatcher(
            api_v2=False, api_v3=True, group='identity-feature-enabled'))
        self._test_validate_service([], self.services, False)

        self.useFixture(fixtures.ConfPatcher(
            api_v2=True, api_v3=True, group='identity-feature-enabled'))
        self._test_validate_service(self.services, self.services, False)

    def test_validate_service_except_invalid_service(self):
        """Negative test case to ensure ``validate_service`` works.

        There are 4 possibilities:
            1) Identity v3 API enabled.
            2) Identity v2 API enabled.
            3) Both are enabled.
            4) Neither are enabled.
        """
        self.useFixture(fixtures.ConfPatcher(
            api_v2=True, api_v3=False, group='identity-feature-enabled'))
        self._test_validate_service(self.services, [], True)

        self.useFixture(fixtures.ConfPatcher(
            api_v2=False, api_v3=True, group='identity-feature-enabled'))
        self._test_validate_service([], self.services, True)

        self.useFixture(fixtures.ConfPatcher(
            api_v2=True, api_v3=True, group='identity-feature-enabled'))
        self._test_validate_service(self.services, self.services, True)

        self.useFixture(fixtures.ConfPatcher(
            api_v2=False, api_v3=False, group='identity-feature-enabled'))
        self._test_validate_service([], [], True, [])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base


class SubnetsRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(SubnetsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('subnet_allocation', 'network'):
            msg = "subnet_allocation extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SubnetsRbacTest, cls).resource_setup()
        cls.network = cls.create_network()
        cls.subnet = cls.create_subnet(cls.network)

    @decorators.idempotent_id('0481adeb-4301-44d5-851c-35910cc18a6b')
    @rbac_rule_validation.action(service="neutron",
                                 rule="create_subnet")
    def test_create_subnet(self):
        """Create subnet.

        RBAC test for the neutron "create_subnet" policy
        """
        with self.rbac_utils.override_role(self):
            self.create_subnet(self.network)

    @decorators.idempotent_id('c02618e7-bb20-4abd-83c8-6eec2af08752')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_subnet")
    def test_show_subnet(self):
        """Show subnet.

        RBAC test for the neutron "get_subnet" policy
        """
        with self.rbac_utils.override_role(self):
            self.subnets_client.show_subnet(self.subnet['id'])

    @decorators.idempotent_id('e2ddc415-5cab-43f4-9b61-166aed65d637')
    @rbac_rule_validation.action(service="neutron",
                                 rule="get_subnet")
    def test_list_subnets(self):
        """List subnets.

        RBAC test for the neutron "get_subnet" policy
        """
        with self.rbac_utils.override_role(self):
            self.subnets_client.list_subnets()

    @decorators.idempotent_id('f36cd821-dd22-4bd0-b43d-110fc4b553eb')
    @rbac_rule_validation.action(service="neutron",
                                 rule="update_subnet")
    def test_update_subnet(self):
        """Update subnet.

        RBAC test for the neutron "update_subnet" policy
        """
        update_name = data_utils.rand_name(self.__class__.__name__ + '-Subnet')

        with self.rbac_utils.override_role(self):
            self.subnets_client.update_subnet(self.subnet['id'],
                                              name=update_name)

    @decorators.idempotent_id('bcfc7153-bbd1-43a4-a908-b3e1b0cde0dc')
    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_subnet")
    def test_delete_subnet(self):
        """Delete subnet.

        RBAC test for the neutron "delete_subnet" policy
        """
        subnet = self.create_subnet(self.network)

        with self.rbac_utils.override_role(self):
            self.subnets_client.delete_subnet(subnet['id'])



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os

from oslo_concurrency import lockutils

from tempest import config
from tempest.test_discover import plugins

from patrole_tempest_plugin import config as project_config

RBACLOG = logging.getLogger('rbac_reporting')


class PatroleTempestPlugin(plugins.TempestPlugin):

    def load_tests(self):
        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "patrole_tempest_plugin/tests/api"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    @lockutils.synchronized('_reset_log_file')
    def _reset_log_file(self, logfile):
        try:
            os.remove(logfile)
        except OSError:
            pass

    def _configure_per_test_logging(self, conf):
        # Separate log handler for rbac reporting
        RBACLOG.setLevel(level=logging.INFO)
        # Set up proper directory handling
        report_abs_path = os.path.abspath(conf.patrole_log.report_log_path)
        report_path = os.path.join(
            report_abs_path, conf.patrole_log.report_log_name)

        # Remove the log file if it exists
        self._reset_log_file(report_path)

        # Delay=True so that we don't end up creating an empty file if we
        # never log to it.
        rbac_report_handler = logging.FileHandler(
            filename=report_path, delay=True, mode='a')
        rbac_report_handler.setFormatter(
            fmt=logging.Formatter(fmt='%(message)s'))
        RBACLOG.addHandler(rbac_report_handler)

    def register_opts(self, conf):
        config.register_opt_group(
            conf,
            project_config.patrole_group,
            project_config.PatroleGroup)
        config.register_opt_group(
            conf,
            project_config.patrole_log_group,
            project_config.PatroleLogGroup)

        if conf.patrole_log.enable_reporting:
            self._configure_per_test_logging(conf)

    def get_opt_lists(self):
        return [(project_config.patrole_group.name,
                 project_config.PatroleGroup)]



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.network import rbac_base as base

CONF = config.CONF


class MeteringLabelRulesRbacTest(base.BaseNetworkRbacTest):

    @classmethod
    def skip_checks(cls):
        super(MeteringLabelRulesRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('metering', 'network'):
            msg = "metering extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(MeteringLabelRulesRbacTest, cls).setup_clients()
        cls.metering_labels_client = cls.os_primary.metering_labels_client
        cls.metering_label_rules_client = \
            cls.os_primary.metering_label_rules_client

    @classmethod
    def resource_setup(cls):
        super(MeteringLabelRulesRbacTest, cls).resource_setup()
        body = cls.metering_labels_client.create_metering_label(
            name=data_utils.rand_name(cls.__name__))
        cls.label = body['metering_label']
        cls.addClassResourceCleanup(
            cls.metering_labels_client.delete_metering_label, cls.label['id'])

    def _create_metering_label_rule(self, label):
        body = self.metering_label_rules_client.create_metering_label_rule(
            metering_label_id=label['id'],
            remote_ip_prefix=CONF.network.project_network_cidr,
            direction="ingress")
        label_rule = body['metering_label_rule']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.metering_label_rules_client.delete_metering_label_rule,
            label_rule['id'])
        return label_rule

    @rbac_rule_validation.action(service="neutron",
                                 rule="create_metering_label_rule")
    @decorators.idempotent_id('81e81776-9d41-4d5e-b5c4-59d5c54a31ad')
    def test_create_metering_label_rule(self):
        """Create metering label rule.

        RBAC test for the neutron create_metering_label_rule policy
        """
        with self.rbac_utils.override_role(self):
            self._create_metering_label_rule(self.label)

    @rbac_rule_validation.action(service="neutron",
                                 rule="get_metering_label_rule",
                                 expected_error_code=404)
    @decorators.idempotent_id('e21b40c3-d44d-412f-84ea-836ca8603bcb')
    def test_show_metering_label_rule(self):
        """Show metering label rule.

        RBAC test for the neutron get_metering_label_rule policy
        """
        label_rule = self._create_metering_label_rule(self.label)
        with self.rbac_utils.override_role(self):
            self.metering_label_rules_client.show_metering_label_rule(
                label_rule['id'])

    @rbac_rule_validation.action(service="neutron",
                                 rule="delete_metering_label_rule",
                                 expected_error_code=404)
    @decorators.idempotent_id('e3adc88c-05c0-43a7-8e32-63947ae4890e')
    def test_delete_metering_label_rule(self):
        """Delete metering label rule.

        RBAC test for the neutron delete_metering_label_rule policy
        """
        label_rule = self._create_metering_label_rule(self.label)
        with self.rbac_utils.override_role(self):
            self.metering_label_rules_client.delete_metering_label_rule(
                label_rule['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class LimitsV3RbacTest(rbac_base.BaseVolumeRbacTest):
    _api_version = 3

    @decorators.idempotent_id('dab04510-5b86-4479-a633-6e496ff405af')
    @rbac_rule_validation.action(service="cinder",
                                 rule="limits_extension:used_limits")
    def test_show_limits(self):
        with self.rbac_utils.override_role(self):
            self.volume_limits_client.show_limits()



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumesTransfersV3RbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def setup_clients(cls):
        super(VolumesTransfersV3RbacTest, cls).setup_clients()
        cls.transfers_client = cls.os_primary.volume_transfers_v2_client

    @classmethod
    def resource_setup(cls):
        super(VolumesTransfersV3RbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()

    def _delete_transfer(self, transfer):
        # Volume from create_volume_transfer test may get stuck in
        # 'awaiting-transfer' state, preventing cleanup and causing
        # the test to fail.
        test_utils.call_and_ignore_notfound_exc(
            self.transfers_client.delete_volume_transfer, transfer['id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, self.volume['id'], 'available')

    def _create_transfer(self):
        transfer = self.transfers_client.create_volume_transfer(
            volume_id=self.volume['id'])['transfer']
        self.addCleanup(self._delete_transfer, transfer)
        return transfer

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:create_transfer")
    @decorators.idempotent_id('25413af4-468d-48ff-94ca-4436f8526b3e')
    def test_create_volume_transfer(self):
        with self.rbac_utils.override_role(self):
            self._create_transfer()

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:get_transfer")
    @decorators.idempotent_id('7a0925d3-ed97-4c25-8299-e5cdabe2eb55')
    def test_get_volume_transfer(self):
        transfer = self._create_transfer()
        with self.rbac_utils.override_role(self):
            self.transfers_client.show_volume_transfer(transfer['id'])

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:get_all_transfers")
    @decorators.idempotent_id('02a06f2b-5040-49e2-b2b7-619a7db59603')
    def test_list_volume_transfers(self):
        with self.rbac_utils.override_role(self):
            self.transfers_client.list_volume_transfers()

    @decorators.idempotent_id('e84e45b0-9872-40bf-bf44-971266161a86')
    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:get_all_transfers")
    def test_list_volume_transfers_details(self):
        with self.rbac_utils.override_role(self):
            self.transfers_client.list_volume_transfers(detail=True)

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:accept_transfer")
    @decorators.idempotent_id('987f2a11-d657-4984-a6c9-28f06c1cd014')
    def test_accept_volume_transfer(self):
        transfer = self._create_transfer()
        with self.rbac_utils.override_role(self):
            self.transfers_client.accept_volume_transfer(
                transfer['id'], auth_key=transfer['auth_key'])

    @rbac_rule_validation.action(service="cinder",
                                 rule="volume:delete_transfer")
    @decorators.idempotent_id('4672187e-7fff-454b-832a-5c8865dda868')
    def test_delete_volume_transfer(self):
        transfer = self._create_transfer()
        with self.rbac_utils.override_role(self):
            self.transfers_client.delete_volume_transfer(transfer['id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, self.volume['id'], 'available')



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base


class VolumeServicesV3RbacTest(rbac_base.BaseVolumeRbacTest):

    # TODO(felipemonteiro): Implement a test to cover the policy action,
    # "volume_extension:services:update", once the Tempest client endpoint
    # is implemented.

    @classmethod
    def skip_checks(cls):
        super(VolumeServicesV3RbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-services', 'volume'):
            msg = "%s skipped as os-services not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(VolumeServicesV3RbacTest, cls).setup_clients()
        cls.services_client = cls.os_primary.volume_services_v2_client

    @decorators.idempotent_id('b9134f01-97c0-4abd-9455-fe2f03e3f966')
    @rbac_rule_validation.action(
        service="cinder",
        rule="volume_extension:services:index")
    def test_list_services(self):
        with self.rbac_utils.override_role(self):
            self.services_client.list_services()['services']



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class FlavorExtraSpecsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(FlavorExtraSpecsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-flavor-extra-specs', 'compute'):
            msg = "os-flavor-extra-specs extension not enabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(FlavorExtraSpecsRbacTest, cls).resource_setup()
        cls.flavor = cls.create_flavor()

    def _set_flavor_extra_spec(self):
        rand_key = data_utils.rand_name(self.__class__.__name__ + '-key')
        rand_val = data_utils.rand_name(self.__class__.__name__ + '-val')
        specs = {rand_key: rand_val}
        self.flavors_client.set_flavor_extra_spec(self.flavor['id'],
                                                  **specs)['extra_specs']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.flavors_client.unset_flavor_extra_spec,
                        self.flavor['id'], rand_key)
        return rand_key

    @decorators.idempotent_id('daee891d-dfe9-4501-a39c-29f2371bec3c')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-flavor-extra-specs:show")
    def test_show_flavor_extra_spec(self):
        key = self._set_flavor_extra_spec()
        with self.rbac_utils.override_role(self):
            self.flavors_client.show_flavor_extra_spec(self.flavor['id'], key)

    @decorators.idempotent_id('fcffeca2-ed04-4e85-bf93-02fb5643f22b')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-flavor-extra-specs:create")
    def test_set_flavor_extra_spec(self):
        with self.rbac_utils.override_role(self):
            self._set_flavor_extra_spec()

    @decorators.idempotent_id('42b85279-6bfa-4f58-b7a2-258c284f03c5')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-flavor-extra-specs:update")
    def test_update_flavor_extra_spec(self):
        key = self._set_flavor_extra_spec()
        update_val = data_utils.rand_name(self.__class__.__name__ + '-val')
        with self.rbac_utils.override_role(self):
            self.flavors_client.update_flavor_extra_spec(
                self.flavor['id'], key, **{key: update_val})

    @decorators.idempotent_id('4b0e5471-e010-4c09-8965-80898e6760a3')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-flavor-extra-specs:delete")
    def test_unset_flavor_extra_spec(self):
        key = self._set_flavor_extra_spec()
        with self.rbac_utils.override_role(self):
            self.flavors_client.unset_flavor_extra_spec(self.flavor['id'], key)

    @decorators.idempotent_id('02c3831a-3ce9-476e-a722-d805ac2da621')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-flavor-extra-specs:index")
    def test_list_flavor_extra_specs(self):
        self._set_flavor_extra_spec()
        with self.rbac_utils.override_role(self):
            self.flavors_client.list_flavor_extra_specs(self.flavor['id'])



#    Copyright 2017 AT&T Corporation.
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base as image_base
from tempest import config

from patrole_tempest_plugin import rbac_utils

CONF = config.CONF


class BaseV2ImageRbacTest(rbac_utils.RbacUtilsMixin,
                          image_base.BaseV2ImageTest):

    @classmethod
    def skip_checks(cls):
        super(BaseV2ImageRbacTest, cls).skip_checks()
        cls.skip_rbac_checks()

    @classmethod
    def setup_clients(cls):
        super(BaseV2ImageRbacTest, cls).setup_clients()
        cls.setup_rbac_utils()



#    Copyright 2017 AT&T Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import testtools

from tempest.lib import exceptions
from tempest import manager
from tempest import test
from tempest.tests import base

from patrole_tempest_plugin import rbac_exceptions
from patrole_tempest_plugin import rbac_rule_validation as rbac_rv
from patrole_tempest_plugin import rbac_utils
from patrole_tempest_plugin.tests.unit import fixtures


class RBACRuleValidationTest(base.TestCase):

    def setUp(self):
        super(RBACRuleValidationTest, self).setUp()
        self.mock_test_args = mock.Mock(spec=test.BaseTestCase)
        self.mock_test_args.os_primary = mock.Mock(spec=manager.Manager)
        self.mock_test_args.rbac_utils = mock.Mock(
            spec_set=rbac_utils.RbacUtils)

        # Setup credentials for mock client manager.
        mock_creds = mock.Mock(user_id=mock.sentinel.user_id,
                               project_id=mock.sentinel.project_id)
        setattr(self.mock_test_args.os_primary, 'credentials', mock_creds)

        self.useFixture(
            fixtures.ConfPatcher(rbac_test_role='Member', group='patrole'))
        # Disable patrole log for unit tests.
        self.useFixture(
            fixtures.ConfPatcher(enable_reporting=False, group='patrole_log'))

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_have_permission_no_exc(self, mock_authority,
                                                    mock_log):
        """Test that having permission and no exception thrown is success.

        Positive test case success scenario.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        test_policy(self.mock_test_args)
        mock_log.warning.assert_not_called()
        mock_log.error.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_lack_permission_throw_exc(self, mock_authority,
                                                       mock_log):
        """Test that having no permission and exception thrown is success.

        Negative test case success scenario.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_policy(self.mock_test_args)
        mock_log.warning.assert_not_called()
        mock_log.error.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_forbidden_negative(self, mock_authority,
                                                mock_log):
        """Test Forbidden error is thrown and have permission fails.

        Negative test case: if Forbidden is thrown and the user should be
        allowed to perform the action, then the Forbidden exception should be
        raised.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise exceptions.Forbidden()

        test_re = "Role Member was not allowed to perform sentinel.action."
        self.assertRaisesRegex(exceptions.Forbidden, test_re, test_policy,
                               self.mock_test_args)
        mock_log.error.assert_called_once_with("Role Member was not allowed to"
                                               " perform sentinel.action.")

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_rbac_malformed_response_positive(
            self, mock_authority, mock_log):
        """Test RbacMalformedResponse error is thrown without permission passes.

        Positive test case: if RbacMalformedResponse is thrown and the user is
        not allowed to perform the action, then this is a success.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise rbac_exceptions.RbacMalformedResponse()

        mock_log.error.assert_not_called()
        mock_log.warning.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_rbac_malformed_response_negative(
            self, mock_authority, mock_log):
        """Test RbacMalformedResponse error is thrown with permission fails.

        Negative test case: if RbacMalformedResponse is thrown and the user is
        allowed to perform the action, then this is an expected failure.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise rbac_exceptions.RbacMalformedResponse()

        test_re = "Role Member was not allowed to perform sentinel.action."
        self.assertRaisesRegex(exceptions.Forbidden, test_re, test_policy,
                               self.mock_test_args)
        mock_log.error.assert_called_once_with("Role Member was not allowed to"
                                               " perform sentinel.action.")

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_rbac_conflicting_policies_positive(
            self, mock_authority, mock_log):
        """Test RbacConflictingPolicies error is thrown without permission passes.

        Positive test case: if RbacConflictingPolicies is thrown and the user
        is not allowed to perform the action, then this is a success.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise rbac_exceptions.RbacConflictingPolicies()

        mock_log.error.assert_not_called()
        mock_log.warning.assert_not_called()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_rbac_conflicting_policies_negative(self,
                                                                mock_authority,
                                                                mock_log):
        """Test RbacConflictingPolicies error is thrown with permission fails.

        Negative test case: if RbacConflictingPolicies is thrown and the user
        is allowed to perform the action, then this is an expected failure.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            raise rbac_exceptions.RbacConflictingPolicies()

        test_re = "Role Member was not allowed to perform sentinel.action."
        self.assertRaisesRegex(exceptions.Forbidden, test_re, test_policy,
                               self.mock_test_args)
        mock_log.error.assert_called_once_with("Role Member was not allowed to"
                                               " perform sentinel.action.")

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_expect_not_found_but_raises_forbidden(self, mock_authority,
                                                   mock_log):
        """Test that expecting 404 but getting 403 works for all scenarios.

        Tests the following scenarios:
        1) Test no permission and 404 is expected but 403 is thrown throws
           exception.
        2) Test have permission and 404 is expected but 403 is thrown throws
           exception.
        """
        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action,
                        expected_error_code=404)
        def test_policy(*args):
            raise exceptions.Forbidden('Test message')

        error_msg = ("An unexpected exception has occurred during test: "
                     "test_policy. Exception was: Forbidden\nDetails: Test "
                     "message")

        for allowed in [True, False]:
            mock_authority.PolicyAuthority.return_value.allowed.\
                return_value = allowed
            self.assertRaisesRegex(exceptions.Forbidden, 'Test message',
                                   test_policy, self.mock_test_args)
            self.assertIn(error_msg, mock_log.error.mock_calls[0][1][0])
            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_expect_not_found_and_raise_not_found(self, mock_authority,
                                                  mock_log):
        """Test that expecting 404 and getting 404 works for all scenarios.

        Tests the following scenarios:
        1) Test no permission and 404 is expected and 404 is thrown succeeds.
        2) Test have permission and 404 is expected and 404 is thrown fails.

        In both cases, a LOG.warning is called with the "irregular message"
        that signals to user that a 404 was expected and caught.
        """
        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action,
                        expected_error_code=404)
        def test_policy(*args):
            raise exceptions.NotFound()

        expected_errors = [
            "Role Member was not allowed to perform sentinel.action.", None
        ]

        for pos, allowed in enumerate([True, False]):
            mock_authority.PolicyAuthority.return_value.allowed\
                .return_value = allowed

            expected_error = expected_errors[pos]

            if expected_error:
                self.assertRaisesRegex(
                    exceptions.Forbidden, '.* ' + expected_error, test_policy,
                    self.mock_test_args)
                mock_log.error.assert_called_once_with(expected_error)
            else:
                test_policy(self.mock_test_args)
                mock_log.error.assert_not_called()

            mock_log.warning.assert_called_once_with(
                "NotFound exception was caught for policy action {0}. The "
                "service {1} throws a 404 instead of a 403, which is "
                "irregular.".format(mock.sentinel.action,
                                    mock.sentinel.service))

            mock_log.warning.reset_mock()
            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rule_validation_overpermission_negative(self, mock_authority,
                                                     mock_log):
        """Test that OverPermission is correctly handled.

        Tests that case where no exception is thrown but the Patrole framework
        says that the role should not be allowed to perform the policy action.
        """
        mock_authority.PolicyAuthority.return_value.allowed.return_value =\
            False

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy_expect_forbidden(*args):
            pass

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action,
                        expected_error_code=404)
        def test_policy_expect_not_found(*args):
            pass

        for test_policy in (
            test_policy_expect_forbidden, test_policy_expect_not_found):

            error_re = (".* OverPermission: Role Member was allowed to perform"
                        " sentinel.action")
            self.assertRaisesRegex(rbac_exceptions.RbacOverPermission,
                                   error_re, test_policy, self.mock_test_args)
            mock_log.error.assert_called_once_with(
                'Role %s was allowed to perform %s', 'Member',
                mock.sentinel.action)
            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_invalid_policy_rule_raises_parsing_exception(
            self, mock_authority):
        """Test that invalid policy action causes test to be fail with
        ``[patrole] strict_policy_check`` set to True.
        """
        self.useFixture(
            fixtures.ConfPatcher(strict_policy_check=True, group='patrole'))

        mock_authority.PolicyAuthority.return_value.allowed.\
            side_effect = rbac_exceptions.RbacParsingException

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        error_re = 'Attempted to test an invalid policy file or action'
        self.assertRaisesRegex(rbac_exceptions.RbacParsingException, error_re,
                               test_policy, self.mock_test_args)

        mock_authority.PolicyAuthority.assert_called_once_with(
            mock.sentinel.project_id, mock.sentinel.user_id,
            mock.sentinel.service, extra_target_data={})

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_invalid_policy_rule_raises_skip_exception(
            self, mock_authority):
        """Test that invalid policy action causes test to be skipped with
        ``[patrole] strict_policy_check`` set to False.
        """
        self.useFixture(
            fixtures.ConfPatcher(strict_policy_check=False, group='patrole'))

        mock_authority.PolicyAuthority.return_value.allowed.side_effect = (
            rbac_exceptions.RbacParsingException)

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        error_re = 'Attempted to test an invalid policy file or action'
        self.assertRaisesRegex(testtools.TestCase.skipException, error_re,
                               test_policy, self.mock_test_args)

        mock_authority.PolicyAuthority.assert_called_once_with(
            mock.sentinel.project_id, mock.sentinel.user_id,
            mock.sentinel.service, extra_target_data={})

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_get_exception_type_404(self, _):
        """Test that getting a 404 exception type returns NotFound."""
        expected_exception = exceptions.NotFound
        expected_irregular_msg = ("NotFound exception was caught for policy "
                                  "action {0}. The service {1} throws a 404 "
                                  "instead of a 403, which is irregular.")

        actual_exception, actual_irregular_msg = \
            rbac_rv._get_exception_type(404)

        self.assertEqual(expected_exception, actual_exception)
        self.assertEqual(expected_irregular_msg, actual_irregular_msg)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_get_exception_type_403(self, _):
        """Test that getting a 404 exception type returns Forbidden."""
        expected_exception = exceptions.Forbidden
        expected_irregular_msg = None

        actual_exception, actual_irregular_msg = \
            rbac_rv._get_exception_type(403)

        self.assertEqual(expected_exception, actual_exception)
        self.assertEqual(expected_irregular_msg, actual_irregular_msg)

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    def test_exception_thrown_when_type_is_not_int(self, mock_log, _):
        """Test that non-integer exception type raises error."""
        self.assertRaises(rbac_exceptions.RbacInvalidErrorCode,
                          rbac_rv._get_exception_type, "403")

        mock_log.error.assert_called_once_with("Please pass an expected error "
                                               "code. Currently supported "
                                               "codes: [403, 404]")

    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    @mock.patch.object(rbac_rv, 'LOG', autospec=True)
    def test_exception_thrown_when_type_is_403_or_404(self, mock_log, _):
        """Test that unsupported exceptions throw error."""
        invalid_exceptions = [200, 400, 500]
        for exc in invalid_exceptions:
            self.assertRaises(rbac_exceptions.RbacInvalidErrorCode,
                              rbac_rv._get_exception_type, exc)
            mock_log.error.assert_called_once_with(
                "Please pass an expected error code. Currently supported "
                "codes: [403, 404]")

            mock_log.error.reset_mock()

    @mock.patch.object(rbac_rv, 'RBACLOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rbac_report_logging_disabled(self, mock_authority, mock_rbaclog):
        """Test case to ensure that we DON'T write logs when  enable_reporting
        is False
        """
        self.useFixture(
            fixtures.ConfPatcher(enable_reporting=False, group='patrole_log'))

        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        test_policy(self.mock_test_args)
        self.assertFalse(mock_rbaclog.info.called)

    @mock.patch.object(rbac_rv, 'RBACLOG', autospec=True)
    @mock.patch.object(rbac_rv, 'policy_authority', autospec=True)
    def test_rbac_report_logging_enabled(self, mock_authority, mock_rbaclog):
        """Test case to ensure that we DO write logs when enable_reporting is
        True
        """
        self.useFixture(
            fixtures.ConfPatcher(enable_reporting=True, group='patrole_log'))

        mock_authority.PolicyAuthority.return_value.allowed.return_value = True

        @rbac_rv.action(mock.sentinel.service, mock.sentinel.action)
        def test_policy(*args):
            pass

        test_policy(self.mock_test_args)
        mock_rbaclog.info.assert_called_once_with(
            "[Service]: %s, [Test]: %s, [Rule]: %s, "
            "[Expected]: %s, [Actual]: %s",
            mock.sentinel.service, 'test_policy',
            mock.sentinel.action,
            "Allowed",
            "Allowed")



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import exceptions


class RbacConflictingPolicies(exceptions.TempestException):
    message = ("Conflicting policies preventing this action from being "
               "performed.")


class RbacMalformedResponse(exceptions.TempestException):
    message = ("The response body is missing the expected %(attribute)s due "
               "to policy enforcement failure.")

    def __init__(self, empty=False, extra_attr=False, **kwargs):
        if empty:
            self.message = ("The response body is empty due to policy "
                            "enforcement failure.")
            kwargs = {}
        if extra_attr:
            self.message = ("The response body contained an unexpected "
                            "attribute due to policy enforcement failure.")
            kwargs = {}
        super(RbacMalformedResponse, self).__init__(**kwargs)


class RbacResourceSetupFailed(exceptions.TempestException):
    message = "Rbac resource setup failed"


class RbacOverPermission(exceptions.TempestException):
    message = "Action performed that should not be permitted"


class RbacInvalidService(exceptions.TempestException):
    message = "Attempted to test an invalid service"


class RbacParsingException(exceptions.TempestException):
    message = "Attempted to test an invalid policy file or action"


class RbacInvalidErrorCode(exceptions.TempestException):
    message = "Unsupported error code passed in test"



# Copyright 2017 NEC Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.volume import rbac_base

CONF = config.CONF


class SnapshotManageRbacTest(rbac_base.BaseVolumeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotManageRbacTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.manage_snapshot:
            raise cls.skipException("Manage snapshot tests are disabled")
        if len(CONF.volume.manage_snapshot_ref) != 2:
            msg = ("Manage snapshot ref is not correctly configured, "
                   "it should be a list of two elements")
            raise lib_exc.InvalidConfiguration(msg)

    @classmethod
    def setup_clients(cls):
        super(SnapshotManageRbacTest, cls).setup_clients()
        cls.snapshot_manage_client = cls.os_primary.snapshot_manage_v2_client

    @classmethod
    def resource_setup(cls):
        super(SnapshotManageRbacTest, cls).resource_setup()
        cls.volume = cls.create_volume()
        cls.snapshot = cls.create_snapshot(volume_id=cls.volume['id'])

    @decorators.idempotent_id('bd7d62f2-e485-4626-87ef-03b7f19ee1d0')
    @rbac_rule_validation.action(
        service="cinder",
        rule="snapshot_extension:snapshot_manage")
    def test_manage_snapshot_rbac(self):
        name = data_utils.rand_name(self.__class__.__name__ +
                                    '-Managed-Snapshot')
        snapshot_ref = {
            'volume_id': self.volume['id'],
            'ref': {CONF.volume.manage_snapshot_ref[0]:
                    CONF.volume.manage_snapshot_ref[1] % self.snapshot['id']},
            'name': name
        }
        with self.rbac_utils.override_role(self):
            snapshot = self.snapshot_manage_client.manage_snapshot(
                **snapshot_ref)['snapshot']
        self.addCleanup(self.delete_snapshot, snapshot['id'],
                        self.snapshots_client)
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                snapshot['id'],
                                                'available')

    @decorators.idempotent_id('4a2e8934-9c0b-434e-8f0b-e18b9aff126f')
    @rbac_rule_validation.action(
        service="cinder",
        rule="snapshot_extension:snapshot_unmanage")
    def test_unmanage_snapshot_rbac(self):
        with self.rbac_utils.override_role(self):
            self.snapshots_client.unmanage_snapshot(self.snapshot['id'])
        self.snapshots_client.wait_for_resource_deletion(
            self.snapshot['id'])



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base

CONF = config.CONF


class FloatingIpsRbacTest(rbac_base.BaseV2ComputeRbacTest):

    # Tests will fail with a 404 starting from microversion 2.36:
    # See the following link for details:
    # https://developer.openstack.org/api-ref/compute/#floating-ips-os-floating-ips-deprecated
    max_microversion = '2.35'

    @classmethod
    def skip_checks(cls):
        super(FloatingIpsRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-floating-ips', 'compute'):
            msg = "%s skipped as os-floating-ips extension not enabled." \
                  % cls.__name__
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @decorators.idempotent_id('ac1b3053-f755-4cda-85a0-30e88b88d7ba')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-floating-ips")
    def test_list_floating_ips(self):
        with self.rbac_utils.override_role(self):
            self.floating_ips_client.list_floating_ips()

    @decorators.idempotent_id('bebe52b3-5269-4e72-80c8-5a4a39c3bfa6')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-floating-ips")
    def test_show_floating_ip(self):
        body = self.floating_ips_client.create_floating_ip(
            pool=CONF.network.floating_network_name)['floating_ip']
        self.addCleanup(
            self.floating_ips_client.delete_floating_ip, body['id'])
        with self.rbac_utils.override_role(self):
            self.floating_ips_client.show_floating_ip(body['id'])

    @decorators.idempotent_id('2bfb8745-c329-4ee9-95f6-c165a1989dbf')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-floating-ips")
    def test_create_floating_ips(self):
        with self.rbac_utils.override_role(self):
            body = self.floating_ips_client.create_floating_ip(
                pool=CONF.network.floating_network_name)['floating_ip']
        self.addCleanup(
            self.floating_ips_client.delete_floating_ip, body['id'])

    @decorators.idempotent_id('d3028373-5027-4e7a-b761-01c515403ecb')
    @rbac_rule_validation.action(
        service="nova",
        rule="os_compute_api:os-floating-ips")
    def test_delete_floating_ip(self):
        body = self.floating_ips_client.create_floating_ip(
            pool=CONF.network.floating_network_name)['floating_ip']
        self.addCleanup(
            test_utils.call_and_ignore_notfound_exc,
            self.floating_ips_client.delete_floating_ip, body['id'])
        with self.rbac_utils.override_role(self):
            self.floating_ips_client.delete_floating_ip(body['id'])



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base


class IdentityAuthV3RbacTest(rbac_base.BaseIdentityV3RbacTest):
    """Tests the APIs that enforce the auth policy actions.

    For more information about the auth policy actions, see:
    https://github.com/openstack/keystone/blob/master/keystone/common/policies/auth.py
    """

    # TODO(felipemonteiro): Add tests for identity:get_auth_catalog
    # once the endpoints are implemented in Tempest's
    # identity v3 client.

    @decorators.idempotent_id('2a9fbf7f-6feb-4161-ae4b-faf7d6421b1a')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_auth_projects")
    def test_list_auth_projects(self):
        with self.rbac_utils.override_role(self):
            self.identity_client.list_auth_projects()

    @decorators.idempotent_id('6a40af0d-7265-4657-b6b2-87a2828e263e')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_auth_domains")
    def test_list_auth_domain(self):
        with self.rbac_utils.override_role(self):
            self.identity_client.list_auth_domains()



#    Copyright 2017 AT&T Corporation.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg


patrole_group = cfg.OptGroup(name='patrole', title='Patrole Testing Options')


PatroleGroup = [
    cfg.StrOpt('rbac_test_role',
               default='admin',
               help="""The current RBAC role against which to run Patrole
tests."""),
    cfg.BoolOpt('enable_rbac',
                default=True,
                help="Enables RBAC tests."),
    cfg.BoolOpt('strict_policy_check',
                default=True,
                deprecated_for_removal=True,
                deprecated_reason="""This option allows for the possibility
of false positives. As a testing framework, Patrole should fail any test that
passes in an invalid policy.""",
                help="""If true, throws RbacParsingException for policies which
don't exist or are not included in the service's policy file. If false, throws
skipException."""),
    # TODO(rb560u): There needs to be support for reading these JSON files from
    # other hosts. It may be possible to leverage the v3 identity policy API.
    cfg.ListOpt('custom_policy_files',
                default=['/etc/%s/policy.json'],
                help="""List of the paths to search for policy files. Each
policy path assumes that the service name is included in the path once. Also
assumes Patrole is on the same host as the policy files. The paths should be
ordered by precedence, with high-priority paths before low-priority paths. The
first path that is found to contain the service's policy file will be used.
"""),
    cfg.BoolOpt('test_custom_requirements',
                default=False,
                help="""
This option determines whether Patrole should run against a
`custom_requirements_file` which defines RBAC requirements. The
purpose of setting this flag to True is to verify that RBAC policy
is in accordance to requirements. The idea is that the
`custom_requirements_file` perfectly defines what the RBAC requirements are.

Here are the possible outcomes when running the Patrole tests against
a `custom_requirements_file`:

YAML definition: allowed
test run: allowed
test result: pass

YAML definition: allowed
test run: not allowed
test result: fail (under-permission)

YAML definition: not allowed
test run: allowed
test result: fail (over-permission)
"""),
    cfg.StrOpt('custom_requirements_file',
               help="""
File path of the yaml file that defines your RBAC requirements. This
file must be located on the same host that Patrole runs on. The yaml
file should be written as follows:

```
<service>:
  <api_action>:
    - <allowed_role>
    - <allowed_role>
    - <allowed_role>
  <api_action>:
    - <allowed_role>
    - <allowed_role>
<service>
  <api_action>:
    - <allowed_role>
```
Where:
service = the service that is being tested (cinder, nova, etc)
api_action = the policy action that is being tested. Examples:
             - volume:create
             - os_compute_api:servers:start
             - add_image
allowed_role = the Keystone role that is allowed to perform the API
""")
]


patrole_log_group = cfg.OptGroup(
    name='patrole_log', title='Patrole Logging Options')

PatroleLogGroup = [
    cfg.BoolOpt('enable_reporting',
                default=False,
                help="Enables reporting on RBAC expected and actual test "
                     "results for each Patrole test"),
    cfg.StrOpt('report_log_name',
               default='patrole.log',
               help="Name of file where output from 'enable_reporting' is "
                    "logged. Note that this file is recreated on each "
                    "invocation of patrole"),
    cfg.StrOpt('report_log_path',
               default='.',
               help="Path (relative or absolute) where the output from "
                    "'enable_reporting' is logged. This is combined with"
                    "report_log_name to generate the full path."),
]


def list_opts():
    """Return a list of oslo.config options available.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users.
    """
    opt_list = [
        (patrole_group, PatroleGroup),
        (patrole_log_group, PatroleLogGroup)
    ]

    return opt_list



# Copyright 2017 AT&T Corporation
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base as compute_base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils

from patrole_tempest_plugin import rbac_utils

CONF = config.CONF


class BaseV2ComputeRbacTest(rbac_utils.RbacUtilsMixin,
                            compute_base.BaseV2ComputeTest):

    @classmethod
    def skip_checks(cls):
        super(BaseV2ComputeRbacTest, cls).skip_checks()
        cls.skip_rbac_checks()

    @classmethod
    def setup_clients(cls):
        super(BaseV2ComputeRbacTest, cls).setup_clients()
        cls.setup_rbac_utils()
        cls.hosts_client = cls.os_primary.hosts_client
        cls.tenant_usages_client = cls.os_primary.tenant_usages_client

    @classmethod
    def create_flavor(cls, **kwargs):
        flavor_kwargs = {
            "name": data_utils.rand_name(cls.__name__ + '-flavor'),
            "ram": data_utils.rand_int_id(1, 10),
            "vcpus": data_utils.rand_int_id(1, 10),
            "disk": data_utils.rand_int_id(1, 10),
            "id": data_utils.rand_uuid(),
        }
        if kwargs:
            flavor_kwargs.update(kwargs)
        flavor = cls.flavors_client.create_flavor(**flavor_kwargs)['flavor']
        cls.addClassResourceCleanup(
            cls.flavors_client.wait_for_resource_deletion, flavor['id'])
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            cls.flavors_client.delete_flavor, flavor['id'])
        return flavor



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.compute import rbac_base


class AggregatesRbacTest(rbac_base.BaseV2ComputeRbacTest):

    @classmethod
    def skip_checks(cls):
        super(AggregatesRbacTest, cls).skip_checks()
        if not utils.is_extension_enabled('os-aggregates', 'compute'):
            msg = "%s skipped as os-aggregates not enabled." % cls.__name__
            raise cls.skipException(msg)

    @classmethod
    def setup_clients(cls):
        super(AggregatesRbacTest, cls).setup_clients()
        cls.hosts_client = cls.os_primary.hosts_client

    def _create_aggregate(self):
        agg_name = data_utils.rand_name(self.__class__.__name__ + '-aggregate')
        aggregate = self.aggregates_client.create_aggregate(name=agg_name)
        aggregate_id = aggregate['aggregate']['id']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.aggregates_client.delete_aggregate,
                        aggregate_id)
        return aggregate_id

    def _add_host_to_aggregate(self, aggregate_id):
        host_name = self.hosts_client.list_hosts()['hosts'][0]['host_name']
        self.aggregates_client.add_host(aggregate_id, host=host_name)
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.aggregates_client.remove_host,
                        aggregate_id,
                        host=host_name)
        return host_name

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:create")
    @decorators.idempotent_id('ba754393-896e-434a-9704-452ff4a84f3f')
    def test_create_aggregate_rbac(self):
        with self.rbac_utils.override_role(self):
            self._create_aggregate()

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:show")
    @decorators.idempotent_id('8fb0b749-b120-4727-b3fb-bcfa3fa6f55b')
    def test_show_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        with self.rbac_utils.override_role(self):
            self.aggregates_client.show_aggregate(aggregate_id)

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:index")
    @decorators.idempotent_id('146284da-5dd6-4c97-b598-42b480f014c6')
    def test_list_aggregate_rbac(self):
        with self.rbac_utils.override_role(self):
            self.aggregates_client.list_aggregates()

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:update")
    @decorators.idempotent_id('c94e0d69-99b6-477e-b301-2cd0e9d0ad81')
    def test_update_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        new_name = data_utils.rand_name(self.__class__.__name__ + '-aggregate')
        with self.rbac_utils.override_role(self):
            self.aggregates_client.update_aggregate(aggregate_id,
                                                    name=new_name)

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:delete")
    @decorators.idempotent_id('5a50c5a6-0f12-4405-a1ce-2288ae895ea6')
    def test_delete_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        with self.rbac_utils.override_role(self):
            self.aggregates_client.delete_aggregate(aggregate_id)

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:add_host")
    @decorators.idempotent_id('97e6e9df-5291-4faa-8147-755b2d1f1ce2')
    def test_add_host_to_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        with self.rbac_utils.override_role(self):
            self._add_host_to_aggregate(aggregate_id)

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:remove_host")
    @decorators.idempotent_id('5b035a25-75d2-4d72-b4d6-0f0337335628')
    def test_remove_host_from_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        host_name = self._add_host_to_aggregate(aggregate_id)
        with self.rbac_utils.override_role(self):
            self.aggregates_client.remove_host(aggregate_id, host=host_name)

    @rbac_rule_validation.action(
        service="nova", rule="os_compute_api:os-aggregates:set_metadata")
    @decorators.idempotent_id('ed6f3849-065c-4ae9-a81e-6ad7ed0d3d9d')
    def test_set_metadata_on_aggregate_rbac(self):
        aggregate_id = self._create_aggregate()
        rand_key = data_utils.rand_name(self.__class__.__name__ + '-key')
        rand_val = data_utils.rand_name(self.__class__.__name__ + '-val')
        with self.rbac_utils.override_role(self):
            self.aggregates_client.set_metadata(
                aggregate_id,
                metadata={rand_key: rand_val})



# Copyright 2017 AT&T Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators

from patrole_tempest_plugin import rbac_rule_validation
from patrole_tempest_plugin.tests.api.identity import rbac_base

CONF = config.CONF


class DomainConfigurationV3RbacTest(rbac_base.BaseIdentityV3RbacTest):
    """RBAC tests for domain configuration client.

    Provides coverage for the following policy actions:
    https://github.com/openstack/keystone/blob/master/keystone/common/policies/domain_config.py
    """

    identity = {"driver": "ldap"}
    ldap = {"url": "ldap://myldap.com:389/",
            "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"}

    @classmethod
    def resource_setup(cls):
        super(DomainConfigurationV3RbacTest, cls).resource_setup()
        cls.domain_id = cls.setup_test_domain()['id']

    def setUp(self):
        super(DomainConfigurationV3RbacTest, self).setUp()
        self._create_domain_config(self.domain_id)

    def _create_domain_config(self, domain_id):
        domain_config = self.domain_config_client.create_domain_config(
            domain_id, identity=self.identity, ldap=self.ldap)['config']
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.domain_config_client.delete_domain_config,
                        domain_id)
        return domain_config

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:create_domain_config")
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd115')
    def test_create_domain_config(self):
        with self.rbac_utils.override_role(self):
            self._create_domain_config(self.domain_id)

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config")
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd118')
    def test_show_domain_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_domain_config(self.domain_id)

    @decorators.idempotent_id('1b539f95-4991-4e09-960f-fa771e1007d7')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config")
    def test_show_domain_group_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_domain_group_config(
                self.domain_id, 'identity')

    @decorators.idempotent_id('590c774d-a294-44f8-866e-aac9f4ab3809')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config")
    def test_show_domain_group_option_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_domain_group_option_config(
                self.domain_id, 'identity', 'driver')

    @decorators.idempotent_id('21053885-1ce3-4167-b5e3-e470253481da')
    @rbac_rule_validation.action(
        service="keystone",
        rule="identity:get_security_compliance_domain_config")
    def test_show_security_compliance_domain_config(self):
        # The "security_compliance" group can only be shown for the default
        # domain.
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_domain_group_config(
                CONF.identity.default_domain_id, 'security_compliance')

    @decorators.idempotent_id('d1addd10-9ae4-4360-9961-47324fd22f23')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config_default")
    def test_show_default_config_settings(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_default_config_settings()

    @decorators.idempotent_id('63183377-251f-4622-81f0-6b58a8a285c9')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config_default")
    def test_show_default_group_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_default_group_config('identity')

    @decorators.idempotent_id('6440e9c1-e8da-474d-9118-89996fffe5f8')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:get_domain_config_default")
    def test_show_default_group_option(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.show_default_group_option('identity',
                                                                'driver')

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_domain_config")
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd116')
    def test_update_domain_config(self):
        updated_config = {'ldap': {'url': data_utils.rand_url()}}
        with self.rbac_utils.override_role(self):
            self.domain_config_client.update_domain_config(
                self.domain_id, **updated_config)

    @decorators.idempotent_id('6e32bf96-dbe9-4ac8-b814-0e79fa948285')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_domain_config")
    def test_update_domain_group_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.update_domain_group_config(
                self.domain_id, 'identity', identity=self.identity)

    @decorators.idempotent_id('d2c510da-a077-4c67-9522-27745ef2812b')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:update_domain_config")
    def test_update_domain_group_option_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.update_domain_group_option_config(
                self.domain_id, 'identity', 'driver', driver='ldap')

    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_domain_config")
    @decorators.idempotent_id('6bdaecd4-0843-4ed6-ab64-3a57ab0cd117')
    def test_delete_domain_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.delete_domain_config(self.domain_id)

    @decorators.idempotent_id('f479694b-df02-4d5a-88b6-c8b52f9341eb')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_domain_config")
    def test_delete_domain_group_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.delete_domain_group_config(
                self.domain_id, 'identity')

    @decorators.idempotent_id('f594bde3-31c9-414f-922d-0ddafdc0ca40')
    @rbac_rule_validation.action(service="keystone",
                                 rule="identity:delete_domain_config")
    def test_delete_domain_group_option_config(self):
        with self.rbac_utils.override_role(self):
            self.domain_config_client.delete_domain_group_option_config(
                self.domain_id, 'identity', 'driver')
